/***************************************************************************
                          fileinfoext.h  -  description
                             -------------------
    begin                : tue dec 9 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef _FILEINFOEXT_H_
#define _FILEINFOEXT_H_

#include "enums.h"

#include <unistd.h>
#include <sys/stat.h>
#include <qfileinfo.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa rozszerzonej informacji o pliku.
 Dostpne jest tu wiele metod udostpniajce rne rozszerzone (wzgldem
 klasy QFileInfo) informacje o pliku. Uywajc tej klasy mona uzyska
 nastpujce informacje:
 @li czy plik ma ustawiony jeden z bitw: UID, GID, VXD;
 @li czy plik to kolejka FIFO, urzdzenie blokowe, urzdzenie znakowe, gniazdko;
 @li rodzaj pliku (typ zgodny z KindOfFile);
 @li nazw pliku bez cieki lub sam ciek bez nazwy;
 @li czy plik to archiwum i typ tego archiwum (zgodny z KindOfArchive)
*/
class FileInfoExt : public QFileInfo
{
public:
	/** Konstruktor klasy.\n
	 Inicjowana jest tu podanym parametrem skadowa, poza tym brak definicji.
	 */
	FileInfoExt( const QString & fileName ) : QFileInfo(fileName), mFileName(fileName) {}
	FileInfoExt() : QFileInfo() {}

	/** Destruktor klasy.\n
	 Brak definicji, poniewa jest zbdna.
	 */
	virtual ~FileInfoExt() {}

	/** Zwraca rozmiar pliku.\n
	 @return rozmiar pliku w bajtach.
	*/
	long long size() { return size( mFileName ); }

// 	QString owner() const { QFileInfo::owner().isEmpty() ? QString::number(ownerId()) : QFileInfo::owner(); }
// 	QString group() const { QFileInfo::group().isEmpty() ? QString::number(groupId()) : QFileInfo::group(); }

	/** Sprawdza czy plik ma ustawiony bit UID, w prawach dostpu.
	 @return TRUE, jeli bit jest ustawiony, w przeciwnym razie FALSE.
	*/
	bool isUID() { return specialPerm( mFileName, UID ); }

	/** Sprawdza czy plik ma ustawiony bit GID, w prawach dostpu.
	 @return TRUE, jeli bit jest ustawiony, w przeciwnym razie FALSE.
	*/
	bool isGID() { return specialPerm( mFileName, GID ); }

	/** Sprawdza czy plik ma ustawiony bit VXT, w prawach dostpu.
	 @return TRUE, jeli bit jest ustawiony, w przeciwnym razie FALSE.
	*/
	bool isVTX() { return specialPerm( mFileName, VTX ); }


	/** Sprawdza czy plik to kolejka FIFO.\n
	 @return TRUE, jeli plik to kolejka FIFO, w przeciwnym razie FALSE.
	*/
	bool isFifo()    { return specialFile( mFileName, FIFO );   }

	/** Sprawdza czy plik to urzdzenie blokowe.\n
	 @return TRUE, jeli plik to urzdzenie blokowe, w przeciwnym razie FALSE.
	*/
	bool isBlkDev()  { return specialFile( mFileName, BLKDEV ); }

	/** Sprawdza czy plik to urzdzenie znakowe.\n
	 @return TRUE, jeli plik to urzdzenie znakowe, w przeciwnym razie FALSE.
	*/
	bool isChrDev()  { return specialFile( mFileName, CHRDEV ); }

	/** Sprawdza czy plik jest gniadkiem (SOCKET).\n
	 @return TRUE, jeli plik to gniazdko, w przeciwnym razie FALSE.
	*/
	bool isSocket()  { return specialFile( mFileName, SOCKET ); }


	/** Funkcja zwraca rodzaj pliku zgodny z type @em KindOfFile dla podanego
	 rozszerzenia.
	 */
	static KindOfFile kindOfFile( const QString & fileName );

	/** Funkcja zwraca ciek do podanej nazwy pliku.
	 @param fileName - nazwa pliku, z ktrego naley pobra ciek,
	 @return cieka z podanego pliku.
	 */
	static QString filePath( const QString & fileName );

	/** Zwraca nazw pliku bez ewentualnie towarzyszcej jej cieki.
	 @param fileName - absolutna nazwa pliku, z ktrego naley pobra nazw,
	 @return nazwa podanego pliku bez cieki.
	 */
	static QString fileName( const QString & fileName );

	/** Funkcja zwraca nazw hosta dla podanego URLa.\n
	 @param fileName - sciezka do pliku lub katalogu
	 @param addSlash - ustawione na TRUE powoduje ze ostatnim znakiem hosta bdzie
	 znak slash ('/').
	 */
	static QString hostName( const QString & url, bool addSlash=TRUE );

	/** Sprawdza czy podany plik to archiwum.\n
	 @param fileName - nazwa pliku do sprawdzenia,
	 @return TRUE jeli nazwa wskazuje na archiwum, w przeciwnym razie FALSE.\n
	 Funkcja przglda ca ciek podanego pliku poszukiwaniu nazwy archiwum
	 */
	static bool isArchive( const QString & fileName );

	/** Zwraca nazw archiwum obcinajc ewentualn ciek za nazw.
	 @param fileName - nazwa pliku archiwum.
	 @return nazwa archiwum
	 */
	static QString archiveFullName( const QString & fileName );

	/** Zwraca rodzaj archiwum zgodny z typem @em KindOfArchive .
	 @param fileName - nazwa pliku archiwum,
	 @return rodzaj archiwum typu @em KindOfArchive .
	 */
	static KindOfArchive kindOfArchive( const QString & fileName );

	/** Zwraca sum praw dostpu do podanego pliku.\n
	 @param fileName - nazwa pliku, ktremu naley pobra prawa dostpu.
	 @return suma praw dostpu.\n
	 Poza standardowymi prawami, sprawdzane s take atrybuty takie jak: S_ISUID,
	 S_ISGID, S_ISVTX. Nazwa nie musi zawiera cieki absolutnej.
	 */
	static int permission( const QString & fileName );

	/** Zwraca wag podanego pliku.\n
	 @param fileName - nazwa pliku, ktremu naley pobra wag.
	 @return waga pliku.
	 */
	static long long size( const QString & fileName );

	/** Typ dla plikw:
	 @li SOCKET - gniazdko,
	 @li BLKDEV - urzdzenie blokowe,
	 @li CHRDEV - urzdzenia znakowego
	 @li FIFO   - kolejka FIFO
	 */
	enum SpecialFileType { NONEsft=0, SOCKET=S_IFSOCK, BLKDEV=S_IFBLK, CHRDEV=S_IFCHR, FIFO=S_IFIFO };

	/** Typ dla plikw ktre maj ustawiony jeden z bitw: UID, GID, VTX.
	 */
	enum SpecialPerm     { NONEsp=0,  UID=S_ISUID, GID=S_ISGID, VTX=S_ISVTX };

	/** Sprawdza czy podany plik zawiera podany bit praw dostpu.\n
	 @param fileName - nazwa pliku, ktry naley sprawdzi,
	 @param sp - bit specjalny praw dostepu zgodny z typem @em SpecialPerm
	 */
	static bool specialPerm( const QString & fileName, SpecialPerm sp );

	/** Sprawdza czy podany plik jest plikiem specjalnym.\n
	 @param fileName - nazwa pliku, ktry naley sprawdzi,
	 @param sft - bit specjalny praw dostepu zgodny z typem @em SpecialFileType
	 */
	static bool specialFile( const QString & fileName, SpecialFileType sft );

private:
	/** Funkcja wykonuje operacje 'stat' na podanym pliku.\n
	 @param fileName - nazwa pliku, ktry naley sprawdzi,
	 @param statInfo - rodzaj bitu do sprawdzenia, dostpne typy to:
	  @em SpecialFileType i @em SpecialPerm .
	 @return TRUE jeli plik posiada podany bit, w przeciwnym razie FALSE.\n
	 Uwaga. Nazwa pliku musi posiada absolutn ciek.
	 */
	static bool doStat( const QString & fileName, int statInfo );


	QString mFileName;

};

#endif
